#include <Arduino.h>
#include "driver/i2s.h"

// Include your converted 16-bit PCM .h files
#include "A4.h"
#include "F5.h"
#include "C5.h"
#include "Db4.h"
#include "Eb5.h"

#define SAMPLE_RATE 16000
#define I2S_NUM I2S_NUM_0

// For analog output using built-in DAC emulation mode
// These are the pins that will output the differential signal
#define I2S_DATA_OUT_PIN D1  // Connect to amplifier input

void setup() {
  Serial.begin(115200);
  
  // Initialize I2S in PDM mode (for analog output without external DAC)
  i2s_config_t i2s_config = {
    .mode = (i2s_mode_t)(I2S_MODE_MASTER | I2S_MODE_TX | I2S_MODE_PDM),
    .sample_rate = SAMPLE_RATE * 2, // PDM requires double sample rate
    .bits_per_sample = I2S_BITS_PER_SAMPLE_16BIT,
    .channel_format = I2S_CHANNEL_FMT_ONLY_LEFT,
    .communication_format = I2S_COMM_FORMAT_STAND_I2S,
    .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1,
    .dma_buf_count = 8,
    .dma_buf_len = 64,
    .use_apll = false,
    .tx_desc_auto_clear = true,
    .fixed_mclk = 0
  };
  
  i2s_pin_config_t pin_config = {
    .bck_io_num = I2S_PIN_NO_CHANGE,
    .ws_io_num = I2S_PIN_NO_CHANGE,
    .data_out_num = I2S_DATA_OUT_PIN,
    .data_in_num = I2S_PIN_NO_CHANGE
  };
  
  i2s_driver_install(I2S_NUM, &i2s_config, 0, NULL);
  i2s_set_pin(I2S_NUM, &pin_config);

  pinMode(D2, INPUT);
  
  Serial.println("I2S PDM initialized");
}

// Function to play a 16-bit PCM sound with full resolution
void playSound(const int16_t* sound, unsigned int length) {
  Serial.println("Playing sound...");
  
  size_t bytes_written;
  
  // Directly write the 16-bit samples to I2S
  i2s_write(I2S_NUM, (const char*)sound, length * 2, &bytes_written, portMAX_DELAY);
  
  // Wait for playback to complete
  i2s_zero_dma_buffer(I2S_NUM);
  
  Serial.println("Finished playing.");
}

void loop() {
  int key = analogRead(D2);

  if(key > 4000){
  playSound(A4_raw, A4_raw_len / 2);  // Divide by 2 if length is in bytes
  delay(2000);
  }
  
  else if(key > 3000){
  playSound(F5_raw, F5_raw_len / 2);
  delay(2000);
  }
  
  else if(key > 2000){
  playSound(C5_raw, C5_raw_len / 2);
  delay(2000);
  }
  
  else if(key > 1500){
  playSound(Db4_raw, Db4_raw_len / 2);
  delay(2000);
  }
  
  else{
  playSound(Eb5_raw, Eb5_raw_len / 2);
  delay(2000);
  }
}
